#include <e_lib.h>
#include "sync.h"

e_mutex_t m SECTION(".data_bank1");

int main(void)
{
	// Open the mailbox and get offsets from it
	unsigned int r, c;
	e_coords_from_coreid(e_get_coreid(), &r, &c);
	unsigned int e_id = (r - E_FIRST_CORE_ROW)*E_COLS_IN_CHIP + (c - E_FIRST_CORE_COL);
	mailbox_t* mailbox = (mailbox_t*)SHARED_DRAM;
	int* done = (int*)DRAM_PTR(mailbox->core_done);

	// Initialize everything if I am core 0, otherwise wait for core 0 to finish
	if (e_id == 0)
	{
		int i;
		for (i = 0; i < 16; i++)
			done[i] = 1;

		mailbox->mutex = e_address_from_coreid(e_get_coreid(), &m);
		e_mutex_init(mailbox->mutex, NULL);

		mailbox->go = 1;
	}
	else
		while (mailbox->go == 0);

	// Increment the counter.  The for loop is there to ensure that, if
	// mutual exclusion fails, all cores will read 0 rather than just one
	// reading 0, resulting in a final value of 1 rather than 16
	e_mutex_lock(mailbox->mutex);
	int local = mailbox->counter;
	volatile int i;
	for (i = 0; i < 1000000; i++);
	mailbox->counter = local + 1;
	e_mutex_unlock(mailbox->mutex);
	done[e_id] = 1;

	// If I am core 0, wait for all other cores to complete, then signal
	// that the chip has finished running
	if (e_id == 0)
	{
		int sum;
		do
		{
			sum = 0;
			int i;
			for (i = 0; i < 16; i++)
				sum += done[i];
		} while (sum < 16);
		mailbox->done = 1;
	}

	return EXIT_SUCCESS;
}
